@@ -1,6 +1,7 @@
 package operator
 
 import (
+	"context"
 	"fmt"
 	"reflect"
 	"strings"
@@ -39,8 +40,8 @@ var (
 )
 
 // statusProgressing sets the Progressing condition to True, with the given
-// reason and message, and sets both the Available and Degraded conditions to
-// False.
+// reason and message, and sets the upgradeable condition to True.  It does not
+// modify any existing Available or Degraded conditions.
 func (optr *Operator) statusProgressing() error {
 	desiredVersions := optr.operandVersions
 	currentVersions, err := optr.getCurrentVersions()
@@ -70,8 +71,6 @@ func (optr *Operator) statusProgressing() error {
 
 	conds := []osconfigv1.ClusterOperatorStatusCondition{
 		newClusterOperatorStatusCondition(osconfigv1.OperatorProgressing, isProgressing, string(ReasonSyncing), message),
-		newClusterOperatorStatusCondition(osconfigv1.OperatorAvailable, osconfigv1.ConditionTrue, "", ""),
-		newClusterOperatorStatusCondition(osconfigv1.OperatorDegraded, osconfigv1.ConditionFalse, "", ""),
 		operatorUpgradeable,
 	}
 
@@ -101,9 +100,8 @@ func (optr *Operator) statusAvailable() error {
 }
 
 // statusDegraded sets the Degraded condition to True, with the given reason and
-// message, and sets the Progressing condition to False, and the Available
-// condition to True.  This indicates that the operator is present and may be
-// partially functioning, but is in a degraded or failing state.
+// message, and sets the upgradeable condition.  It does not modify any existing
+// Available or Progressing conditions.
 func (optr *Operator) statusDegraded(error string) error {
 	desiredVersions := optr.operandVersions
 	currentVersions, err := optr.getCurrentVersions()
@@ -122,8 +120,6 @@ func (optr *Operator) statusDegraded(error string) error {
 	conds := []osconfigv1.ClusterOperatorStatusCondition{
 		newClusterOperatorStatusCondition(osconfigv1.OperatorDegraded, osconfigv1.ConditionTrue,
 			string(ReasonSyncFailed), message),
-		newClusterOperatorStatusCondition(osconfigv1.OperatorProgressing, osconfigv1.ConditionFalse, "", ""),
-		newClusterOperatorStatusCondition(osconfigv1.OperatorAvailable, osconfigv1.ConditionTrue, "", ""),
 		operatorUpgradeable,
 	}
 
@@ -154,31 +150,14 @@ func (optr *Operator) syncStatus(co *osconfigv1.ClusterOperator, conds []osconfi
 		v1helpers.SetStatusCondition(&co.Status.Conditions, c)
 	}
 
-	_, err := optr.osClient.ConfigV1().ClusterOperators().UpdateStatus(co)
+	_, err := optr.osClient.ConfigV1().ClusterOperators().UpdateStatus(context.Background(), co, metav1.UpdateOptions{})
 	return err
 }
 
-func (optr *Operator) getOrCreateClusterOperator() (*osconfigv1.ClusterOperator, error) {
-	var co *osconfigv1.ClusterOperator
-	co, err := optr.osClient.ConfigV1().ClusterOperators().Get(clusterOperatorName, metav1.GetOptions{})
-	if errors.IsNotFound(err) {
-		co = &osconfigv1.ClusterOperator{
-			ObjectMeta: metav1.ObjectMeta{
-				Name: clusterOperatorName,
-			},
-		}
-		glog.Infof("clusterOperator %q does not exist, creating a new one: %v", clusterOperatorName, co)
-		co, err = optr.osClient.ConfigV1().ClusterOperators().Create(co)
-		if err != nil {
-			return nil, fmt.Errorf("failed to create new clusterOperator: %v", err)
-		}
-	}
-	if err != nil {
-		return nil, fmt.Errorf("failed to get clusterOperator %q: %v", clusterOperatorName, err)
-	}
-
-	// Related objects lets openshift/must-gather collect diagnostic content
-	relatedObjects := []osconfigv1.ObjectReference{
+// relatedObjects returns the current list of ObjectReference's for the
+// ClusterOperator objects's status.
+func (optr *Operator) relatedObjects() []osconfigv1.ObjectReference {
+	return []osconfigv1.ObjectReference{
 		{
 			Group:    "",
 			Resource: "namespaces",
@@ -212,20 +191,122 @@ func (optr *Operator) getOrCreateClusterOperator() (*osconfigv1.ClusterOperator,
 			Resource: "clusterroles",
 			Name:     "machine-api-controllers",
 		},
-		{
-			Group:     "rbac.authorization.k8s.io",
-			Resource:  "roles",
-			Name:      "cloud-provider-config-reader",
-			Namespace: "openshift-config",
+	}
+}
+
+// defaultStatusConditions returns the default set of status conditions for the
+// ClusterOperator resource used on first creation of the ClusterOperator.
+func (optr *Operator) defaultStatusConditions() []osconfigv1.ClusterOperatorStatusCondition {
+	// All conditions default to False with no message.
+	return []osconfigv1.ClusterOperatorStatusCondition{
+		newClusterOperatorStatusCondition(
+			osconfigv1.OperatorProgressing,
+			osconfigv1.ConditionFalse,
+			"", "",
+		),
+		newClusterOperatorStatusCondition(
+			osconfigv1.OperatorDegraded,
+			osconfigv1.ConditionFalse,
+			"", "",
+		),
+		newClusterOperatorStatusCondition(
+			osconfigv1.OperatorAvailable,
+			osconfigv1.ConditionFalse,
+			"", "",
+		),
+	}
+}
+
+// defaultClusterOperator returns the default ClusterOperator resource with
+// default values for related objects and status conditions.
+func (optr *Operator) defaultClusterOperator() *osconfigv1.ClusterOperator {
+	return &osconfigv1.ClusterOperator{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: clusterOperatorName,
+		},
+		Status: osconfigv1.ClusterOperatorStatus{
+			Conditions:     optr.defaultStatusConditions(),
+			RelatedObjects: optr.relatedObjects(),
 		},
 	}
+}
+
+// updateRelatedObjects updates the ClusterOperator's related objects field if
+// necessary and returns the updated ClusterOperator object.
+func (optr *Operator) updateRelatedObjects(co *osconfigv1.ClusterOperator) (*osconfigv1.ClusterOperator, error) {
+	relatedObjects := optr.relatedObjects()
+
 	if !equality.Semantic.DeepEqual(co.Status.RelatedObjects, relatedObjects) {
 		co.Status.RelatedObjects = relatedObjects
-		return optr.osClient.ConfigV1().ClusterOperators().UpdateStatus(co)
+		return optr.osClient.ConfigV1().ClusterOperators().UpdateStatus(context.Background(), co, metav1.UpdateOptions{})
 	}
+
 	return co, nil
 }
 
+// setMissingStatusConditions checks that the given ClusterOperator has a value
+// for each of the default status conditions, and sets the default value for any
+// that are missing.
+func (optr *Operator) setMissingStatusConditions(co *osconfigv1.ClusterOperator) (*osconfigv1.ClusterOperator, error) {
+	var modified bool
+
+	for _, c := range optr.defaultStatusConditions() {
+		if v1helpers.FindStatusCondition(co.Status.Conditions, c.Type) == nil {
+			v1helpers.SetStatusCondition(&co.Status.Conditions, c)
+			modified = true
+		}
+	}
+
+	if modified {
+		return optr.osClient.ConfigV1().ClusterOperators().UpdateStatus(context.Background(), co, metav1.UpdateOptions{})
+	}
+
+	return co, nil
+}
+
+// getClusterOperator returns the current ClusterOperator.
+func (optr *Operator) getClusterOperator() (*osconfigv1.ClusterOperator, error) {
+	return optr.osClient.ConfigV1().ClusterOperators().
+		Get(context.Background(), clusterOperatorName, metav1.GetOptions{})
+}
+
+// createClusterOperator creates the ClusterOperator and updates its status.
+func (optr *Operator) createClusterOperator() (*osconfigv1.ClusterOperator, error) {
+	defaultCO := optr.defaultClusterOperator()
+
+	co, err := optr.osClient.ConfigV1().ClusterOperators().Create(context.Background(), defaultCO, metav1.CreateOptions{})
+	if err != nil {
+		return nil, err
+	}
+
+	co.Status = defaultCO.Status
+
+	return optr.osClient.ConfigV1().ClusterOperators().UpdateStatus(context.Background(), co, metav1.UpdateOptions{})
+}
+
+// getOrCreateClusterOperator fetches the current ClusterOperator or creates a
+// default one if not found -- ensuring the related objects list is current.
+func (optr *Operator) getOrCreateClusterOperator() (*osconfigv1.ClusterOperator, error) {
+	existing, err := optr.getClusterOperator()
+
+	if errors.IsNotFound(err) {
+		glog.Infof("ClusterOperator does not exist, creating a new one.")
+		return optr.createClusterOperator()
+	}
+
+	if err != nil {
+		return nil, fmt.Errorf("failed to get clusterOperator %q: %v", clusterOperatorName, err)
+	}
+
+	// Update any missing status conditions with their default value.
+	existing, err = optr.setMissingStatusConditions(existing)
+	if err != nil {
+		return nil, fmt.Errorf("failed to set default conditions: %v", err)
+	}
+
+	return optr.updateRelatedObjects(existing)
+}
+
 func (optr *Operator) getCurrentVersions() ([]osconfigv1.OperandVersion, error) {
 	co, err := optr.getOrCreateClusterOperator()
 	if err != nil {
